# generate three sliders and send value to Arduino 
# Arduino commands: R<n>, G<n>, B<n>, H<n> with <n> as integer number
# 20120610

import kivy
kivy.require('1.0.9') 

import socket

try:
    import cPickle as pickle
except:
    import pickle

from kivy.clock import Clock
from kivy.core.window import Window
from kivy.app import App
from kivy.uix.button import Button
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.gridlayout import GridLayout
from kivy.uix.label import Label
from kivy.uix.switch import Switch
from kivy.uix.floatlayout import FloatLayout
from kivy.uix.widget import Widget
from kivy.factory import Factory
from kivy.lang import Builder
from kivy.properties import NumericProperty, ReferenceListProperty, ObjectProperty
from kivy.properties import ListProperty
from kivy.graphics import RenderContext
from kivy.properties import StringProperty

host = "192.168.0.81"      # remote machine name
port = 80                  # port number

MAX_LED_VALUE = 127        # maximum LED value to send to Arduino

# This header must be not changed, it contain the minimum information from Kivy.
header = '''
#ifdef GL_ES
precision highp float;
#endif

/* Outputs from the vertex shader */
varying vec4 frag_color;
varying vec2 tex_coord0;

/* uniform texture samplers */
uniform sampler2D texture0;
'''

# Plasma shader
plasma_shader = header + '''
uniform vec2 resolution;
uniform float time;
uniform vec4 color_pos;
uniform int color;

void main(void)
{
   float x = gl_FragCoord.x;
   float y = gl_FragCoord.y;
   float c1 = 0.0;
   float c2 = 0.0;
   float c3 = 0.0;
   float width = resolution.x;

    /* generate shaded color bars for R,G,B,H */
    if (x < width/4.0)
        {
        if (y < color_pos.x)
                c1 = y / resolution.y;                 
        }
    else if ( x < 2.0*width/4.0 )
        {
        if (y < color_pos.y)
                c2 = y / resolution.y;                 
        }
    else if ( x < 3.0*width/4.0 )
        {
        if (y < color_pos.z)
                c3 = y / resolution.y;                 
        }
    else
        {
        if (y < color_pos.w)
                c1 = c2 = c3 = y / resolution.y;                 
        }

   gl_FragColor = vec4( c1,c2,c3,1.0);
}
'''

class ColorValues:
    ''' class to store light values '''
    Rled = 50        # red LED value 0...MAX_LED_VALUE
    Gled = 50        # green LED value 0...MAX_LED_VALUE
    Bled = 50        # blue LED value 0...MAX_LED_VALUE
    Hled = 50        # brightness LED value 0...MAX_LED_VALUE
    Rled_old = 0     # copy to check for changes
    Gled_old = 0     # ...
    Bled_old = 0    
    Hled_old = 0    
    MAXCOLOR = 500   # maximum unscaled value (depends on graphics) 

class Communicate:
    '''class for communication with Arduino'''      
    connected = False    
        
    def connect(self):
        ''' connect to Arduino and set LEDs to default level '''
        try:
            self.conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.conn.connect((host, port))
        except socket.error, msg: 
            print "Error connecting"
            self.connected = False
            return
        self.connected = True
        self.send('R',vals.Rled)
        self.send('G',vals.Gled)
        self.send('B',vals.Bled)
        
    def disconnect(self):
        ''' disconnect from Arduino '''
        if self.connected == True:
            try:
                self.conn.close()
            except socket.error, msg: 
                print "Error disconnecting"
        self.connected = False
    
    def send(self,color,value):
        ''' set new LED value and check for connection error '''
        if self.connected == True:
            st = color + str(int(value)) + "\n"
            try:
                self.conn.send(st)
            except socket.error, msg:
                print "Error sending"
                self.connected = False

    def initial_colorbars(self):
        ''' set initial values for colorbar '''
        colors = [
            vals.Rled * vals.MAXCOLOR / MAX_LED_VALUE,
            vals.Gled * vals.MAXCOLOR / MAX_LED_VALUE,
            vals.Bled * vals.MAXCOLOR / MAX_LED_VALUE,
            vals.Hled * vals.MAXCOLOR / MAX_LED_VALUE ]
        return colors
        
    def update_values(self,colors):
        ''' update LED values from graphics '''
        vals.Rled = min(MAX_LED_VALUE,max(0,int(colors[0]  * MAX_LED_VALUE / vals.MAXCOLOR)))
        vals.Gled = min(MAX_LED_VALUE,max(0,int(colors[1]  * MAX_LED_VALUE / vals.MAXCOLOR)))
        vals.Bled = min(MAX_LED_VALUE,max(0,int(colors[2]  * MAX_LED_VALUE / vals.MAXCOLOR)))
        vals.Hled = min(MAX_LED_VALUE,max(0,int(colors[3]  * MAX_LED_VALUE / vals.MAXCOLOR)))
        
    def process_touch(self,colors):        
        ''' check if LED values need to be sent to Arduino '''
        self.update_values(colors)
        if vals.Rled_old != vals.Rled:
            self.send('R',vals.Rled)
            vals.Rled_old=vals.Rled
        if vals.Gled_old != vals.Gled:
            self.send('G',vals.Gled)
            vals.Gled_old=vals.Gled
        if vals.Bled_old != vals.Bled: 
            self.send('B',vals.Bled)
            vals.Bled_old=vals.Bled
        if vals.Hled_old != vals.Hled:
            self.send('H',vals.Hled)
            vals.Hled_old=vals.Hled
    
class Connected(Widget):
    '''display if connected to Arduino webserver as red or green dot'''
    red_color =  NumericProperty(1)
    green_color =  NumericProperty(0)
    def set_color(self,color):
        if color == True:
            self.red_color = 0
            self.green_color = 1
            comm.connect()
        else:
            self.red_color = 1
            self.green_color = 0
            comm.disconnect()

class ColorSlider(Widget):
    ''' Handle color slider'''
    color_pos = [0,0,0,0]
    old_colors = [0,0,0,0]
    control = ObjectProperty(None)

    fs = StringProperty(None)
    def __init__(self, **kwargs):
        ''' set shader for canvas'''
        self.canvas = RenderContext()
        super(ColorSlider, self).__init__(fs=plasma_shader)
        Clock.schedule_interval(self.update_glsl, 1 / 60.)
        self.color_pos = comm.initial_colorbars()
        self.old_colors = self.color_pos

    def on_fs(self, instance, value):
        shader = self.canvas.shader
        old_value = shader.fs
        shader.fs = value
        print shader.fs
        if not shader.success:
            shader.fs = old_value
            raise Exception('failed')

    def update_glsl(self, *largs):
        ''' update colors for shader '''
        self.canvas['resolution'] =  map(float, self.size)
        self.canvas['color_pos'] =   map(float, self.color_pos)
        self.canvas['projection_mat'] = Window.render_context['projection_mat']
        vals.MAXCOLOR = self.height
               
    def on_touch_down(self, touch):
        ''' touch event to change value + 1 or - 1 '''
        if touch.y < self.height:
            if touch.y < self.height/2:
                delta = -vals.MAXCOLOR / MAX_LED_VALUE
            else:
                delta = vals.MAXCOLOR / MAX_LED_VALUE
            if touch.x < self.width/4:
                self.color_pos[0] = self.color_pos[0]+delta  # red
            elif touch.x< self.width*2/4:
                self.color_pos[1] = self.color_pos[1] +delta   # green
            elif touch.x< self.width*3/4:
                self.color_pos[2] = self.color_pos[2] +delta   # blue           
            else:
                self.color_pos[3] = self.color_pos[3] +delta   # brightness
            comm.process_touch(self.color_pos)
            self.control.update_values()
        
    def on_touch_move(self, touch):        
        if touch.y < self.height:
            ''' update color values after touch and move event'''
            color_val = min(touch.y, vals.MAXCOLOR) 
            if touch.x < self.width/4:
                self.color_pos[0] = color_val   # red
            elif touch.x< self.width*2/4:
                self.color_pos[1] = color_val   # green
            elif touch.x< self.width*3/4:
                self.color_pos[2] = color_val   # blue
            else:
                self.color_pos[3] = color_val   # brightnes
            comm.process_touch(self.color_pos)
            self.control.update_values()

class ColorControl(Widget):
    '''Main widget'''
    switch = ObjectProperty(None)
    red_value =  StringProperty('Red')
    green_value =  StringProperty('Green')
    blue_value =  StringProperty('Blue')
    bright_value =  StringProperty('Bright')
    rgbslider = ObjectProperty(None)
    labelstring = ObjectProperty(None)
    labelcolor= (1.0,0,1.0,1.0)
    
    def setup(self):    
        self.switch.bind(active=self.callback)
        self.rgbslider.control = self   # required to call update_values from ColorSlider
        self.update_values()
    
    def callback(self,instance,value):
        ''' callback for turning connection on/off '''
        self.connected.set_color(value)

    def update_values(self):
        ''' display current color values '''
        br = 1.0*vals.Hled/MAX_LED_VALUE # brightness
        self.namestring.color= (br*vals.Rled/MAX_LED_VALUE,br*vals.Gled/MAX_LED_VALUE,br*vals.Bled/MAX_LED_VALUE,1 )
        self.red_value   = "Red = %3.0f" % vals.Rled
        self.green_value = "Green = %3.0f" % vals.Gled
        self.blue_value  = "Blue = %3.0f" % vals.Bled
        self.bright_value  = "Bright = %3.0f" % vals.Hled      
        
        if comm.connected == False:     
            self.connected.set_color(False)
            self.switch.action = False

Factory.register("Connected", Connected)
Factory.register("ColorSlider", ColorSlider)
Factory.register("ColorControl", ColorControl)

class RGBSlider(App):
    def build(self):
        c = ColorControl()
        c.setup()
        return c

if __name__ in ('__android__', '__main__'):  
    try:
        ''' Load settings '''
        f = open("rgbleds.ini", "rb")
        vals_str = f.read()
        vals = pickle.loads(vals_str)
    except:
        vals = ColorValues()    # storage for color values
    comm=Communicate()          # communication class
    RGBSlider().run()
    
    ''' Save settings '''
    f = open("rgbleds.ini", "wb")
    comm_str = pickle.dumps(vals)
    f.write(comm_str)
    f.close()
    
    